package org.jarbframework.violation.factory;

import org.jarbframework.violation.CheckFailedException;
import org.jarbframework.violation.DatabaseConstraintViolation;
import org.jarbframework.violation.DatabaseConstraintViolationException;
import org.jarbframework.violation.ForeignKeyViolationException;
import org.jarbframework.violation.InvalidTypeException;
import org.jarbframework.violation.LengthExceededException;
import org.jarbframework.violation.NotNullViolationException;
import org.jarbframework.violation.UniqueKeyViolationException;

/**
 * Provides a type specific constraint violation exception.
 * 
 * @author Jeroen van Schagen
 * @since 12-05-2011
 */
public class SimpleConstraintExceptionFactory implements DatabaseConstraintExceptionFactory {

    /**
     * {@inheritDoc}
     */
    @Override
    public DatabaseConstraintViolationException createException(DatabaseConstraintViolation violation, Throwable cause) {
        DatabaseConstraintViolationException exception = null;
        switch (violation.getViolationType()) {
        case NOT_NULL:
            exception = new NotNullViolationException(violation, cause);
            break;
        case CHECK_FAILED:
            exception = new CheckFailedException(violation, cause);
            break;
        case INVALID_TYPE:
            exception = new InvalidTypeException(violation, cause);
            break;
        case LENGTH_EXCEEDED:
            exception = new LengthExceededException(violation, cause);
            break;
        case UNIQUE_KEY:
            exception = new UniqueKeyViolationException(violation, cause);
            break;
        case FOREIGN_KEY:
            exception = new ForeignKeyViolationException(violation, cause);
            break;
        }
        return exception;
    }

}
